@@ -17,6 +17,9 @@ from pathlib import Path
 
 from .pyaml.libaml.aml import AML, ResTypes, ResXMLTree_attribute, ResourceRef
 
+def plba(filename, arch):
+    p = Path(filename)
+    return f"{os.path.dirname(p)}/{p.stem}_{arch}.so"
 
 class Patcher:
 
@@ -44,11 +47,14 @@ class Patcher:
 
         '''
         self.apk = apk
+        self.arch = None
         self.sdktools = sdktools
         self.version = version
         self.final_dir = None
+        self._pause = False
         self.path_build_tools = f"{sdktools}/build-tools/{version}/" if \
                 (sdktools and version) else ""
+        self.network_certificates = []
 
 ################################################################################
 #                                                                              #
@@ -56,6 +62,61 @@ class Patcher:
 #                                                                              #
 ################################################################################
    
+    def add_network_certificate(self, cert):
+        self.network_certificates.append(cert)
+
+    def inject_custom_network_certificate(self):
+        if len(self.network_certificates) == 0:
+            return
+        res_path = os.path.join(self.final_dir, 'res')
+        xml_path = os.path.join(res_path, 'xml')
+        netsec_path = os.path.join(xml_path, 'network_security_config.xml')
+        shutil.copyfile(os.path.dirname(os.path.abspath(__file__)) + "/network_security_config_custom.xml", netsec_path)
+
+
+        os.makedirs(f"{res_path}/raw", exist_ok=True)
+        for cert in self.network_certificates:
+            shutil.copyfile(cert, f"{res_path}/raw/my_ca")
+        logging.info("Custom certificate was injected inside the apk")
+        #with open(netsec_path, 'rb') as fp:
+        #    buf = fp.read()
+        #    print(buf)
+        #    s.find(b'user')
+        #    aml = AML(buf)
+        #    while aml.hasnext():
+        #        header, body = aml.next()
+                #if header.type == ResTypes.RES_XML_START_ELEMENT_TYPE and \
+                #        body.nodename == 'trust-anchors':
+                #    print(body.nodename)
+                #    for cert in self.network_certificates:
+                #        name_cert = Path(cert).stem
+                #        shutil.copyfile(cert, f"{res_path}/raw/my_ca/{name_cert}")
+                #        cert = aml.insert()
+                #        #cert.writexmlstartelement('certificates',
+                #        #        {'src':f"@raw/{name_cert}"}, ns=None)
+                #        #cert.writexmlendelement('certificates')
+                #replace_user = True
+                #if header.type == ResTypes.RES_XML_START_ELEMENT_TYPE and \
+                #        body.nodename == 'certificates':
+                #    for i in body.attributes:
+                #        if str(i) == 'src':
+                #            if not replace_user:
+                #                for cert in self.network_certificates:
+                #                    name_cert = Path(cert).stem
+                #                    index = i.typedValue._stringpool.strings.index("user")
+                #                    i.typedValue._stringpool.strings[index] = f"{res_path}/raw/my_ca/{name_cert}"
+                #                    replace_user = True
+                #                    #print(i.typedValue.__dict__)
+                #                    #i.typedValue._stringpool._append(f"@raw/{name_cert}")
+                #                    #print(i.typedValue._stringpool.strings)
+                #                    #i.typedValue.data = f"@raw/{name_cert}"
+                #                    #print(i.typedValue.data)
+
+        #with open(netsec_path, 'wb') as fp:
+        #    fp.write(aml.tobytes())
+        #    print(aml.tobytes())
+
+
     def enable_user_certificates(self):
         #if not self.has_user_certificates_label():
         self.inject_user_certificates_label()
@@ -223,12 +284,12 @@ class Patcher:
                 header, body = aml.next()
                 if header.type == ResTypes.RES_XML_START_ELEMENT_TYPE and body.nodename == 'manifest':
                     inserted = aml.insert()
-                    inserted.writexmlstartelement('uses-premission',
+                    inserted.writexmlstartelement('uses-permission',
                         {
                             "name" : permission
                         })
-                    inserted.writexmlendelement('uses-premission',{})
-                    break
+                    inserted.writexmlendelement('uses-permission',{})
+                    #break
 
         with open(manifest_path, 'wb') as fp:
             fp.write(aml.tobytes())
@@ -274,12 +335,15 @@ class Patcher:
         tmp_target_file = apk_path.replace('.apk', '_tmp.apk')
         shutil.move(apk_path, tmp_target_file)
 
-        subprocess.call(f"{self.path_build_tools}zipalign 4 {tmp_target_file} {apk_path}", stderr=subprocess.STDOUT, shell=True)
+        subprocess.call(f"{self.path_build_tools}zipalign -p 4 {tmp_target_file} {apk_path}", stderr=subprocess.STDOUT, shell=True)
 
         os.remove(tmp_target_file)
 
         logging.info('The file was optimized!')
 
+    def pause(self, pause):
+        self._pause = pause
+
     def repackage_apk(self, target_file=None):
         '''
         repackage the apk
@@ -288,6 +352,9 @@ class Patcher:
                     - target_file (str) : the path of the new apk created if
                       none, a new apk will be created with suffix "_patched.apk"
         '''
+        if self._pause:
+            print(f"You can modify the apk here: {self.final_dir}")
+            input()
         if target_file is None:
             current_path = os.getcwd()
             target_file = os.path.join(current_path, self.apk.replace('.apk', '_patched.apk'))
@@ -412,7 +479,7 @@ class Patcher:
 
                 if has_class_constructor and class_constructor_end_index == -1:
                     logging.error('Could not find the end of class constructor.')
-                return False
+                    return False
 
             prologue_start_index = -1
             if has_class_constructor:
@@ -429,7 +496,7 @@ class Patcher:
 
                 if no_prologue_case and locals_start_index == -1:
                     logging.error('Has class constructor. No prologue case, but no "locals 0" found.')
-                return False
+                    return False
 
             locals_end_index = -1
             if no_prologue_case:
@@ -470,7 +537,7 @@ class Patcher:
 
         return True
 
-    def create_lib_arch_folders(self):
+    def create_lib_arch_folders(self, arch):
         '''
         make lib folder in the apk to put native lib
         '''
@@ -484,17 +551,17 @@ class Patcher:
             logging.info('There is no "lib" folder. Creating...')
             os.makedirs(libs_path)
 
-        if self.arch == self.ARCH_ARM:
+        if arch == self.ARCH_ARM:
             sub_dir = os.path.join(libs_path, 'armeabi')
             sub_dir_2 = os.path.join(libs_path, 'armeabi-v7a')
 
-        elif self.arch == self.ARCH_ARM64:
+        elif arch == self.ARCH_ARM64:
             sub_dir = os.path.join(libs_path, 'arm64-v8a')
 
-        elif self.arch == self.ARCH_X86:
+        elif arch == self.ARCH_X86:
             sub_dir = os.path.join(libs_path, 'x86')
 
-        elif self.arch == self.ARCH_X64:
+        elif arch == self.ARCH_X64:
             sub_dir = os.path.join(libs_path, 'x86_64')
 
         else:
@@ -505,25 +572,49 @@ class Patcher:
             logging.info('Creating folder {0}'.format(sub_dir))
             os.makedirs(sub_dir)
 
-        if self.arch == self.ARCH_ARM:
+        if arch == self.ARCH_ARM:
             if not os.path.isdir(sub_dir_2):
                 logging.info('Creating folder {0}'.format(sub_dir_2))
                 os.makedirs(sub_dir_2)
 
-        if self.arch == self.ARCH_ARM:
+        if arch == self.ARCH_ARM:
             return [sub_dir, sub_dir_2]
 
         else:
             return [sub_dir]
 
-    def insert_frida_lib(self, gadget_path : str, config_file_path=None, auto_load_script_path=None):
+    def check_libextract(self):
+        logging.info('check if lib is extractable')
+
+        manifest_path = os.path.join(self.final_dir, 'AndroidManifest.xml')
+
+        if not os.path.isfile(manifest_path):
+            logging.error("Couldn't find the Manifest file. Something is wrong with the apk!")
+            return False
+
+
+        with open(manifest_path, 'rb') as fp:
+            aml = AML(fp.read())
+            while aml.hasnext():
+                header, body = aml.next()
+                if header.type == ResTypes.RES_XML_START_ELEMENT_TYPE and body.nodename == 'application':
+                    for i in body.attributes:
+                        if str(i) == 'android:extractNativeLibs':
+                            logging.info("Added lib extraction enable")
+                            i.typedValue.data = True
+
+        with open(manifest_path, 'wb') as fp:
+            fp.write(aml.tobytes())
+
+
+    def insert_frida_lib(self, gadget_path : str, arch : str, config_file_path=None, auto_load_script_path=None):
         '''
         Insert native lib inside the apk
 
         Parameters:
                     - gadget_path (str): the path of the gadget to insert
         '''
-        arch_folders = self.create_lib_arch_folders()
+        arch_folders = self.create_lib_arch_folders(arch)
 
         if not arch_folders:
             logging.error('Some error occurred while creating the libs folders')
@@ -596,11 +687,16 @@ class Patcher:
 ################################################################################
 
 
-    def patching(self, gadget_to_use, arch, output_file=None, user_certificate=False):
+    def set_arch(self, arch):
+        self.arch = arch
+
+
+    def patching(self, gadget_to_use, output_file=None, user_certificate=False):
         '''
         patch the apk with gadget 'gadget_to_use'
         '''
-        self.arch = arch
+        if len(self.network_certificates) > 0:
+            user_certificate = True
         if not os.path.isfile(self.apk):
             logging.error(f"The file {self.apk} couldn't be found!")
             sys.exit(1)
@@ -622,11 +718,13 @@ class Patcher:
             # add users certificate
             if user_certificate:
                 self.enable_user_certificates()
+                self.inject_custom_network_certificate()
             
             # inject frida library
             self.entrypoint_class = self.get_entrypoint_class_name()
             self.entrypoint_smali_path = self.get_entrypoint_smali_path()
             self.insert_frida_loader()
+            self.check_libextract()
 
             #if args.autoload_script:
             #    if not patcher.min_frida_version('10.6.33'):
@@ -646,7 +744,15 @@ class Patcher:
             #                             config_file_path=default_config_file, auto_load_script_path=script_file)
 
             #else:
-            self.insert_frida_lib(gadget_to_use)
+            if not self.arch:
+                archs = [(plba(gadget_to_use, self.ARCH_ARM),  self.ARCH_ARM),
+                        (plba(gadget_to_use, self.ARCH_ARM64), self.ARCH_ARM64),
+                        (plba(gadget_to_use, self.ARCH_X86),   self.ARCH_X86),
+                        (plba(gadget_to_use, "x86_64"),   self.ARCH_X64)]
+            else:
+                archs = [(gadget_to_use, self.arch)]
+            for gadget, arch in archs:
+                self.insert_frida_lib(gadget, arch)
 
             # repackage the apk and sign + align it
             if output_file:
@@ -699,15 +805,20 @@ def main():
 
     parser.add_argument('-e', '--enable-user-certificates', help='Add some configs in apk to accept user certificates',
                         action='store_true')
+    parser.add_argument('-c', '--custom-certificate', help='Install a custom network certificate inside the apk')
 
     parser.add_argument('-o', '--output-file', help='Specify the output file (patched)')
+    
+    parser.add_argument('-p', '--pause', help='pause before repackage the apk',
+            action="store_true")
 
     args = parser.parse_args()
 
     if len(sys.argv) == 1 or not (args.apk and \
                                   args.sdktools and \
-                                  args.version_buildtools and \
-                                  args.arch):
+                                  args.version_buildtools and
+                                  args.gadget):
+        print("apkpatcher -a <apk> -s <sdktools> -b <version> -g <lib.so> [options]")
         parser.print_help()
         return 1
 
@@ -722,12 +833,17 @@ def main():
         logging.basicConfig(level=logging.INFO)
 
     patcher = Patcher(args.apk, args.sdktools, args.version_buildtools)
+    if args.custom_certificate:
+        patcher.add_network_certificate(args.custom_certificate)
+    if args.arch:
+        patcher.set_arch(args.arch)
+    patcher.pause(args.pause)
     if args.output_file:
-        patcher.patching(args.gadget, args.arch,
+        patcher.patching(args.gadget,
                 output_file=args.output_file,
                 user_certificate=args.enable_user_certificates)
     else:
-        patcher.patching(args.gadget, args.arch,
+        patcher.patching(args.gadget,
                 user_certificate=args.enable_user_certificates)
 
 
