@@ -531,7 +531,7 @@ class AML:
         def tobytes(self):
             return self._bytebuffer.tobytes()
 
-        def writexmlstartelement(self, name, attrs, linenumber=0):
+        def writexmlstartelement(self, name, attrs, linenumber=0, ns='http://schemas.android.com/apk/res/android'):
             stringpool = self._aml.stringpool
             self._aml.stringpool.ensure(name)
             attrExt = ResXMLTree_attrExt.create(ResourceRef.create(AML.NONE_NAMESPACE_REF, stringpool=stringpool),
@@ -540,7 +540,7 @@ class AML:
             element = ResXMLTree.create(ResChunk.Header.create(ResTypes.RES_XML_START_ELEMENT_TYPE, 16, 0),
                                         ResXMLTree_node.create(linenumber or self._node.lineNumber, 0xffffffff),
                                         attrExt, aml=self)
-            androidns = ResourceRef(stringpool=stringpool, value=AML.ANDROID_NAMESPACE)
+            androidns = ResourceRef(stringpool=stringpool, value=ns)
             for k, v in attrs.items():
                 stringpool.setattribute(k, v)
                 attr = ResXMLTree_attribute.make(androidns, stringpool,
